@@ -130,14 +130,34 @@ class XBRLElement:
                     refs[cell.ref] = DateParser.parse(cell.ref)
         return refs
 
+    def ids(self):
+        """
+        Recursive function to access all uri label pairs
+        :return: A dictionary where keys are uri strings and values are label strings or None is there is no label
+        """
+        ids = {self.uri: self.label}
+        for child in self._children:
+            ids.update(child.ids())
+        return ids
+
+    def to_list(self) -> list:
+        """
+        Recursive function to return a list of all elements in the tree
+        :return: A list of each XBRLElement in the tree
+        """
+        lst = [self]
+        for child in self._children:
+            lst.extend(child.to_list())
+        return lst
+
     def to_dict(self) -> dict:
         """
         A recursive function to return a dictionary representation of the tree from this point downward
         :return: A dictionary where keys are labels and cells are bottem level xbrl elements
         """
-        dic = {self.label: []}
+        dic = {self.uri: []}
         if all(not child._children for child in self._children.keys()):
-            dic[self.label] = [ele for ele in self._children.keys()]
+            dic[self.uri] = [ele for ele in self._children.keys()]
         else:
             for ele, o in sorted(self._children.items(), key=lambda item: item[1] or -1):
                 dic.update(ele.to_dict())
@@ -149,8 +169,6 @@ class XBRLAssembler:
     The main object to compile XBRL documents into complete sets of data by establishing a tree of information
 
     Args:
-        :param info: A printable representation of the dataset.
-        :type info: str
         :param schema: A class:`bs4.BeautifulSoup` for the xbrl schema file
         :param data: A class:`bs4.BeautifulSoup` for the xbrl data file
         :param label: A class:`bs4.BeautifulSoup` for the xbrl label file
@@ -158,10 +176,8 @@ class XBRLAssembler:
     """
     uri_re = re.compile(r'(?:lab_)?((us-gaap|source|dei|[a-z]{3,4})[_:][A-Za-z]{5,})', re.IGNORECASE)
 
-    def __init__(self, info, schema, data, label, ref):
+    def __init__(self, schema, data, label, ref):
         """Constructor Method"""
-        self.info = info
-
         for t, name in [(schema, "schema"),
                         (data, "data"),
                         (label, "label"),
@@ -208,8 +224,7 @@ class XBRLAssembler:
             file_map[XBRLType.get(row[3].text)] = soup
 
         try:
-            return cls(info=index_url,
-                       schema=file_map[XBRLType.SCHEMA],
+            return cls(schema=file_map[XBRLType.SCHEMA],
                        data=file_map[XBRLType.DATA],
                        label=file_map[XBRLType.LAB],
                        ref=file_map[ref_doc])
@@ -237,8 +252,7 @@ class XBRLAssembler:
                 file_map[XBRLType.get(item)] = BeautifulSoup(open(os.path.join(directory, item), 'r'), 'lxml')
 
         try:
-            return cls(info=directory,
-                       schema=file_map[XBRLType.SCHEMA],
+            return cls(schema=file_map[XBRLType.SCHEMA],
                        data=file_map[XBRLType.DATA],
                        label=file_map[XBRLType.LAB],
                        ref=file_map[ref_doc])
@@ -271,7 +285,8 @@ class XBRLAssembler:
                 label = roletype.find("link:definition").text
                 if "Parenthetical" not in label:  # "Statement" in label and
                     text = label.split(" - ")
-                    docs[uri] = XBRLElement(uri=uri, label=text[-1], ref=text[0])
+                    ele = XBRLElement(uri=uri, label=text[-1], ref=text[0])
+                    docs[uri] = ele
 
             if not docs:
                 raise AttributeError
@@ -363,9 +378,10 @@ class XBRLAssembler:
         elif isinstance(search, FinancialStatement):
             doc_ele = self.find_doc(lambda name: re.search(search.value, name))
         else:
-            raise ValueError(f"")
+            raise ValueError(f"XBRLAssembler.get() search term should be "
+                             f"re.Pattern, string, or FinancialStatement not {search}")
 
-        if not doc_ele:
+        if doc_ele is None:
             raise XBRLError(f"No match found for {search} in names.\n\t"
                             f"Names available {[name for name in self._docs.keys()]}]")
 
@@ -378,30 +394,33 @@ class XBRLAssembler:
         # Find desired section in reference document
         def_link = self.ref.find(re.compile(r'link', re.IGNORECASE), attrs={'xlink:role': doc_ele.uri})
         if not def_link:
-            raise XBRLError(self.info)
+            raise XBRLError(f"Can't find document in reference doc")
+
 
         # Pull all elements and create XBRLElements out of them
         eles = {}
+        cols = collections.defaultdict(int)
         for loc in def_link.find_all(re.compile(r'loc', re.IGNORECASE)):
             uri = loc['xlink:href'].split('#')[1]
             label = self._labels[uri.lower()] if uri.lower() in self._labels else None
             ele = XBRLElement(uri=uri, label=label)
             eles[loc['xlink:label']] = ele
 
+            if ele.uri.lower() in self._cells:
+                for cell in self._cells[ele.uri.lower()]:
+                    cols[cell.ref] += 1
+
+        if not cols:
+            raise XBRLError(f'{doc_ele.label} could not find any columns for cells')
+
         # Find and create parent/child relationships between new elements
         for arc in def_link.find_all(re.compile(r'\w*arc', re.IGNORECASE)):
             parent, child, order = eles[arc['xlink:from']], eles[arc['xlink:to']], arc['order']
             parent.add_child(child=child, order=order)
 
-        # Remove columns that are not used
-        cols = collections.defaultdict(int)
-        for ele in eles.values():
-            if ele.uri.lower() in self._cells:
-                for cell in self._cells[ele.uri.lower()]:
-                    cols[cell.ref] += 1
-
         most_used = max(cols.values())
-        cols = set([c for c, count in cols.items() if count == most_used])
+        cols = {DateParser.parse(ref): ref for ref, count in cols.items() if count == most_used}
+        cols = set(cols.values())
 
         # Determine top and bottom level elements in the document and either fill in cells or
         #   link them to the overall document element
